import mariadb
import json
from pathlib import Path
from time import perf_counter
from pprint import pprint
from itertools import chain, repeat

conn = mariadb.connect(
       user="root",
       password="root",
       host="localhost",
       port=3306,
    )

c = conn.cursor()
c.execute('USE imdb')

c.execute('SELECT id FROM person')
person_PKs = set(chain.from_iterable(c.fetchall()))

with open('enums.json', encoding='utf-8') as enums_f:
    enums = json.load(enums_f)
for key in enums:  # transform to lookup table for ids: enums['genre']['Comedy'] => genre.id
    enums[key] = {name: i for i, name in enumerate(enums[key], 1)}


def idstrip(idstr, prefix_len=2):
    return int(idstr.strip()[prefix_len:])

def id_helper(parts, column, col_name, prefix_len):
    return idstrip(parts[column[col_name]], prefix_len)

def value_helper(parts, column, col_name, cls):
    cv = parts[column[col_name]].strip()
    return cls(cv) if cv != r'\N' else None

def enum_helper(parts, column, col_name, enum_name):
    v = value_helper(parts, column, col_name, str)
    return enums[enum_name][v] if v is not None else None

# valhelp and idhelp uses the parts and column directly from current scope
valhelp = lambda col_name, cls=str: value_helper(parts, column, col_name, cls)
idhelp = lambda  col_name, prefix_len=2: id_helper(parts, column, col_name, prefix_len)
enmhelp = lambda col_name, enum_name: enum_helper(parts, column, col_name, enum_name)


#c.execute('TRUNCATE TABLE person') # (only for repeated cell running in notebook)
#c.execute('TRUNCATE TABLE person_has_profession') # (only for repeated cell running in notebook)
#c.execute('TRUNCATE TABLE known_for') # (only for repeated cell running in notebook)
t1 = perf_counter()
with open(Path('data') / 'name.basics.tsv', encoding='utf-8') as f:
    column = {}
    for i, p in enumerate(f.readline().strip().split('\t')):
        column[p.strip()] = i
    for li, line in enumerate(f):
        parts = line.strip().split('\t')
        id_ = idhelp('nconst')  # ommit prefix
        if id_ in person_PKs:
            continue
        name = valhelp('primaryName')
        print(id_, name)
        birth = valhelp('birthYear', int)
        death = valhelp('deathYear', int)
        c.execute('INSERT INTO person (id, name, birth_year, death_year) VALUES (?,?,?,?)', (id_, name, birth, death))
        glist = parts[column['primaryProfession']].strip().split(',')
        if len(glist) and glist[0] != '\\N':
            values = tuple(chain.from_iterable((id_, enums['profession'][p]) for p in glist if len(p)))
            if len(values):
                qmarks = ','.join(repeat('(?,?)', len(values) // 2))
                c.execute(f'INSERT INTO person_has_profession (person_id, profession_id) VALUES {qmarks}', values)
        glist = parts[column['knownForTitles']].strip().split(',')
        if len(glist) and glist[0] != '\\N':
            values = tuple(chain.from_iterable((id_, idstrip(p)) for p in glist if len(p)))
            qmarks = ','.join(repeat('(?,?)', len(values) // 2))
            c.execute(f'INSERT INTO known_for (person_id, title_id) VALUES {qmarks}', values)
print(perf_counter() - t1, 's')


